import { Directive, HostBinding, Input, NgModule } from '@angular/core';
import { DomSanitizer } from '@angular/platform-browser';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
const /** @type {?} */ isJsons = (array) => Array.isArray(array) &&
    array.every(row => typeof row === 'object' && !(row instanceof Array));
const /** @type {?} */ isArrays = (array) => Array.isArray(array) && array.every(row => Array.isArray(row));
/**
 * @param {?} array
 * @return {?}
 */
function jsonsHeaders(array) {
    return Array.from(new Set(array.map(item => Object.keys(item)).reduce((a, b) => [...a, ...b], [])));
}
/**
 * @param {?} jsons
 * @param {?=} headers
 * @return {?}
 */
function jsons2arrays(jsons, headers) {
    headers = headers || jsonsHeaders(jsons);
    // allow headers to have custom labels, defaulting to having the header data key be the label
    let /** @type {?} */ headerLabels = /** @type {?} */ (headers);
    let /** @type {?} */ headerKeys = /** @type {?} */ (headers);
    if (isJsons(headers)) {
        headerLabels = (/** @type {?} */ (headers)).map(header => header.label);
        headerKeys = (/** @type {?} */ (headers)).map(header => header.key);
    }
    const /** @type {?} */ data = jsons.map(object => headerKeys.map(header => (header in object ? object[header] : '')));
    return [headerLabels, ...data];
}
const /** @type {?} */ elementOrEmpty = (element) => element || element === 0 ? element : '';
/**
 * @param {?} data
 * @param {?=} delimiter
 * @return {?}
 */
function joiner(data, delimiter = ',') {
    return data
        .map((row, index) => row.map((element) => '"' + elementOrEmpty(element) + '"').join(delimiter))
        .join(`\n`);
}
/**
 * @param {?} data
 * @param {?=} headers
 * @param {?=} delimiter
 * @return {?}
 */
function arrays2csv(data, headers, delimiter) {
    return joiner(headers ? [headers, ...data] : data, delimiter);
}
/**
 * @param {?} data
 * @param {?=} headers
 * @param {?=} delimiter
 * @return {?}
 */
function jsons2csv(data, headers, delimiter) {
    return joiner(jsons2arrays(data, headers), delimiter);
}
/**
 * @param {?} data
 * @param {?=} headers
 * @param {?=} delimiter
 * @return {?}
 */
function string2csv(data, headers, delimiter) {
    return headers ? `${headers.join(delimiter)}\n${data}` : data;
}
/**
 * @param {?} data
 * @param {?=} headers
 * @param {?=} delimiter
 * @return {?}
 */
function toCSV(data, headers, delimiter) {
    if (isJsons(/** @type {?} */ (data))) {
        return jsons2csv(/** @type {?} */ (data), headers, delimiter);
    }
    if (isArrays(/** @type {?} */ (data))) {
        return arrays2csv(/** @type {?} */ (data), headers, delimiter);
    }
    if (typeof data === 'string') {
        return string2csv(data, /** @type {?} */ (headers), delimiter);
    }
    throw new TypeError(`Data should be a "String", "Array of arrays" OR "Array of objects" `);
}
/**
 * @param {?} data
 * @param {?=} uFEFF
 * @param {?=} headers
 * @param {?=} delimiter
 * @return {?}
 */
function buildURI(data, uFEFF = true, headers, delimiter) {
    const /** @type {?} */ csv = toCSV(data, headers, delimiter);
    const /** @type {?} */ blob = new Blob([uFEFF ? '\uFEFF' : '', csv], { type: 'text/csv' });
    return URL.createObjectURL(blob);
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class CsvDirective {
    /**
     * @param {?} sanitizer
     */
    constructor(sanitizer) {
        this.sanitizer = sanitizer;
        /**
         * the body of the csv
         */
        this.data = [];
        /**
         * Set the seperator between values
         */
        this.delimiter = ',';
        /**
         * adds a Byte order mark to setup the csv as UTF-8
         */
        this.uFEFF = true;
        /**
         * filename
         */
        this.download = 'data.csv';
        this.target = '_blank';
    }
    /**
     * Set the filename of the csv. Default is `data.csv`
     * @param {?} a
     * @return {?}
     */
    set filename(a) {
        this.download = a;
    }
    /**
     * @return {?}
     */
    ngOnChanges() {
        this.href = this.sanitizer.bypassSecurityTrustResourceUrl(buildURI(this.data, this.uFEFF, this.headers, this.delimiter));
    }
}
CsvDirective.decorators = [
    { type: Directive, args: [{ selector: 'a[csvLink]' },] },
];
/** @nocollapse */
CsvDirective.ctorParameters = () => [
    { type: DomSanitizer, },
];
CsvDirective.propDecorators = {
    "data": [{ type: Input },],
    "headers": [{ type: Input },],
    "delimiter": [{ type: Input },],
    "filename": [{ type: Input },],
    "uFEFF": [{ type: Input },],
    "href": [{ type: HostBinding },],
    "download": [{ type: HostBinding },],
    "target": [{ type: Input }, { type: HostBinding },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class CsvModule {
}
CsvModule.decorators = [
    { type: NgModule, args: [{
                declarations: [CsvDirective],
                exports: [CsvDirective],
            },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

export { CsvModule, CsvDirective };
//# sourceMappingURL=ctrl-ngx-csv.js.map
